#ifndef _KGR_PATCH_BSC1039496_LOCAL_H
#define _KGR_PATCH_BSC1039496_LOCAL_H

#include <linux/errno.h>
#include <linux/types.h>
#include <linux/static_key.h>

struct user_arg_ptr {
#ifdef CONFIG_COMPAT
	bool is_compat;
#endif
	union {
		const char __user *const __user *native;
#ifdef CONFIG_COMPAT
		const compat_uptr_t __user *compat;
#endif
	} ptr;
};

struct proc_maps_private {
	struct pid *pid;
	struct task_struct *task;
#ifdef CONFIG_MMU
	struct vm_area_struct *tail_vma;
#endif
#ifdef CONFIG_NUMA
	struct mempolicy *task_mempolicy;
#endif
};

struct mm_slot {
	struct hlist_node hash;
	struct list_head mm_node;
	struct mm_struct *mm;
};

struct khugepaged_scan {
	struct list_head mm_head;
	struct mm_slot *mm_slot;
	unsigned long address;
};

#define kgr_pgd_offset_k(address) pgd_offset(kgr_init_mm, (address))
#define kgr_alloc_page_vma(gfp_mask, vma, addr)			\
	kgr_alloc_pages_vma(gfp_mask, 0, vma, addr, numa_node_id(), false)
#define kgr__alloc_zeroed_user_highpage(movableflags, vma, vaddr) \
	kgr_alloc_page_vma(GFP_HIGHUSER | __GFP_ZERO | movableflags, vma, vaddr)

#if defined(CONFIG_DEBUG_VM_RB)
#error "kGraft patch supports only !CONFIG_DEBUG_VM_RB"
#endif
#define kgr_validate_mm(mm) do { } while (0)

#define kgr_free_mm(mm)	(kmem_cache_free(*kgr_mm_cachep, (mm)))

/* trace_sched_process_exec */
#define PARAMS(args...) args
#define TP_PROTO(args...)	args
#define TP_ARGS(args...)	args
#define TP_CONDITION(args...)	args

#undef __DECLARE_TRACE
#define __DECLARE_TRACE(name, proto, args, cond, data_proto, data_args) \
	extern struct tracepoint *kgr__tracepoint_##name;			\
	static inline void kgr_trace_##name(proto)				\
	{								\
		if (unlikely(static_key_count(&kgr__tracepoint_##name->key) > 0))		\
			__DO_TRACE(kgr__tracepoint_##name,		\
				TP_PROTO(data_proto),			\
				TP_ARGS(data_args),			\
				TP_CONDITION(cond),,);			\
	}								\
	__DECLARE_TRACE_RCU(name, PARAMS(proto), PARAMS(args),		\
		PARAMS(cond), PARAMS(data_proto), PARAMS(data_args))	\
	static inline int						\
	kgr_register_trace_##name(void (*probe)(data_proto), void *data)	\
	{								\
		return tracepoint_probe_register(#name, (void *)probe,	\
						 data);			\
	}								\
	static inline int						\
	kgr_unregister_trace_##name(void (*probe)(data_proto), void *data)	\
	{								\
		return tracepoint_probe_unregister(#name, (void *)probe, \
						   data);		\
	}								\
	static inline void						\
	kgr_check_trace_callback_type_##name(void (*cb)(data_proto))	\
	{								\
	}

#undef TRACE_EVENT
#define TRACE_EVENT(name, proto, args, struct, assign, print)	\
	DECLARE_TRACE(name, PARAMS(proto), PARAMS(args))

TRACE_EVENT(sched_process_exec,

	TP_PROTO(struct task_struct *p, pid_t old_pid,
		 struct linux_binprm *bprm),

	TP_ARGS(p, old_pid, bprm),

	TP_STRUCT__entry(
		__string(	filename,	bprm->filename	)
		__field(	pid_t,		pid		)
		__field(	pid_t,		old_pid		)
	),

	TP_fast_assign(
		__assign_str(filename, bprm->filename);
		__entry->pid		= p->pid;
		__entry->old_pid	= old_pid;
	),

	TP_printk("filename=%s pid=%d old_pid=%d", __get_str(filename),
		  __entry->pid, __entry->old_pid)
);

#endif
